@@ -3,7 +3,7 @@ package Shipwright::Source::Base;
 use warnings;
 use strict;
 use Carp;
-use File::Spec::Functions qw/catfile catdir/;
+use File::Spec;
 use File::Slurp;
 use Module::CoreList;
 use Shipwright::Source;
@@ -12,19 +12,10 @@ use Cwd qw/getcwd/;
 
 use base qw/Class::Accessor::Fast/;
 __PACKAGE__->mk_accessors(
-    qw/source directory scripts_directory download_directory follow
-      min_perl_version map_path skip map keep_recommends keep_build_requires
-      name log url_path version_path version/
+    qw/source directory download_directory follow min_perl_version map_path
+      skip map keep_recommends keep_build_requires name log url_path/
 );
 
-=head1 NAME
-
-Shipwright::Source::Base - base class for source
-
-=head1 SYNOPSIS
-
-=head1 METHODS
-
 =head2 new
 
 =cut
@@ -45,12 +36,7 @@ sub run {
     my $self = shift;
     my %args = @_;
     for ( $self->_cmd ) {
-        if ( ref $_ eq 'CODE' ) {
-            $_->();
-        }
-        else {
-            Shipwright::Util->run($_);
-        }
+        Shipwright::Util->run($_);
     }
     $self->_copy( %{ $args{copy} } ) if $args{copy};
 }
@@ -62,9 +48,8 @@ sub _follow {
     my $self         = shift;
     my $path         = shift;
     my $cwd          = getcwd;
-    my $require_path = catfile( $path, '__require.yml' );
+    my $require_path = File::Spec->catfile( $path, '__require.yml' );
     my $map          = {};
-    my $url          = {};
 
     unless ( $self->min_perl_version ) {
         no warnings 'once';
@@ -78,161 +63,74 @@ sub _follow {
         $map = Shipwright::Util::LoadFile( $self->map_path );
     }
 
-    if ( -e $self->url_path ) {
-        $url = Shipwright::Util::LoadFile( $self->url_path );
-    }
-
     if ( !-e $require_path ) {
 
         # if not found, we'll create one according to Build.PL or Makefile.PL
         my $require = {};
-        chdir catdir($path);
+        chdir File::Spec->catfile($path);
 
         if ( -e 'Build.PL' ) {
             Shipwright::Util->run( [ $^X, 'Build.PL' ] );
-            my $source = read_file( catfile( '_build', 'prereqs' ) )
-              or confess "can't read _build/prereqs: $!";
-            my $eval = '$require = ' . $source;
-            eval $eval or confess "eval error: $@";    ## no critic
+            my $source = read_file( File::Spec->catfile( '_build', 'prereqs' ) )
+              or die "can't read _build/prereqs: $!";
+            my $eval .= '$require = ' . $source;
+            eval $eval or die "eval error: $@";    ## no critic
 
-            $source = read_file( catfile('Build.PL') )
-              or confess "can't read Build.PL: $!";
+            $source = read_file( File::Spec->catfile('Build.PL') )
+              or die "can't read Build.PL: $!";
             if (   $source =~ /Module::Build/
                 && $self->name ne 'cpan-Module-Build' )
             {
-                unless ( defined $require->{build_requires}{'Module::Build'} ) {
-                    $require->{build_requires}{'Module::Build'} = 0;
+                unless ( $require->{build_requires}{'Module::Build'} ) {
+                    $require->{build_requires} = { 'Module::Build' => 0 };
                 }
             }
 
             Shipwright::Util->run( [ './Build', 'realclean' ] );
         }
         elsif ( -e 'Makefile.PL' ) {
-            my $makefile = read_file('Makefile.PL')
-              or confess "can't read Makefile.PL: $!";
-
-            if ( $makefile =~ /inc::Module::Install/ ) {
-
-  # PREREQ_PM in Makefile is not good enough for inc::Module::Install, which
-  # will omit features(..). we'll put deps in features(...) into recommends part
-
-                $makefile =~ s/^\s*requires(?!\w)/shipwright_requires/mg;
-                $makefile =~
-                  s/^\s*build_requires(?!\w)/shipwright_build_requires/mg;
-                $makefile =~ s/^\s*features(?!\w)/shipwright_features/mg;
-                my $shipwright_makefile = <<'EOF';
-my $shipwright_req = {};
-
-sub _shipwright_requires {
-    my $type = shift;
-    my %req  = @_;
-    for my $name ( keys %req ) {
-        $shipwright_req->{$type}{$name} = $req{$name};
-    }
-}
-
-sub shipwright_requires {
-    _shipwright_requires( 'requires', @_ == 1 ? ( @_, 0 ) : @_ );
-    goto &requires;
-}
-
-sub shipwright_build_requires {
-    _shipwright_requires( 'build_requires', @_ == 1 ? ( @_, 0 ) : @_ );
-    goto &build_requires;
-}
-
-sub shipwright_features {
-    my @args = @_;
-    while ( my ( $name, $mods ) = splice( @_, 0, 2 ) ) {
-        for ( my $i = 0; $i < @$mods; $i++ ) {
-            if ( $mods->[$i] eq '-default' ) {
-                $i++;
-                next;
+            Shipwright::Util->run( [ $^X, 'Makefile.PL' ] );
+            my ($source) = grep { /PREREQ_PM/ } read_file('Makefile');
+            if ( $source && $source =~ /({.*})/ ) {
+                my $eval .= '$require = ' . $1;
+                $eval =~ s/([\w:]+)=>/'$1'=>/g;
+                eval $eval or die "eval error: $@";    ## no critic
             }
 
-            if ( ref $mods->[$i] eq 'ARRAY' ) {
-# this happends when
-# features(
-#     'Date loading' => [
-#         -default => 0,
-#        recommends( 'DateTime' )
-#     ],
-# );
-               for ( my $j = 0; $j < @{$mods->[$i]}; $j++ ) {
-                    if ( $mods->[$i][$j+1] =~ /^[\d\.]+$/ ) {
-                        $shipwright_req->{recommends}{$mods->[$i][$j]} 
-                            = $mods->[$i][$j+1];
-                        $j++;
-                    }
-                    else {
-                        $shipwright_req->{recommends}{$mods->[$i][$j]} = 0;
-                    }
-                }
-                
-                next;
+            for ( keys %$require ) {
+                $require->{requires}{$_} = delete $require->{$_};
             }
 
-            if ( $mods->[$i+1] =~ /^[\d\.]+$/ ) {
-                # index $i+1 is a version
-                $shipwright_req->{recommends}{$mods->[$i]} = $mods->[$i+1];
-                $i++;
-            }
-            else {
-                $shipwright_req->{recommends}{$mods->[$i]} = 0;
-            }
-        }
-    }
-    
-    goto &features;
-}
+            $source = read_file('Makefile.PL')
+              or die "can't read Makefile.PL: $!";
 
-END {
-require Data::Dumper;
-open my $tmp_fh, '>', 'shipwright_prereqs';
-print $tmp_fh Data::Dumper->Dump( [$shipwright_req], [qw/require/] );
-}
-
-EOF
-
-                $shipwright_makefile .= $makefile;
-                write_file( 'shipwright_makefile.pl', $shipwright_makefile );
-
-                Shipwright::Util->run( [ $^X, 'shipwright_makefile.pl' ] );
-                my $prereqs = read_file( catfile('shipwright_prereqs') )
-                  or confess "can't read prereqs: $!";
-                eval $prereqs or confess "eval error: $@";    ## no critic
-
-                Shipwright::Util->run( [ 'rm', 'shipwright_makefile.pl' ] );
-                Shipwright::Util->run( [ 'rm', 'shipwright_prereqs' ] );
-            }
-            else {
-
-                # we extract the deps from Makefile
-                Shipwright::Util->run( [ $^X, 'Makefile.PL' ] );
-                my ($source) = grep { /PREREQ_PM/ } read_file('Makefile');
-                if ( $source && $source =~ /({.*})/ ) {
-                    my $eval .= '$require = ' . $1;
-                    $eval =~ s/([\w:]+)=>/'$1'=>/g;
-                    eval $eval or confess "eval error: $@";    ## no critic
-                }
-
-                for ( keys %$require ) {
-                    $require->{requires}{$_} = delete $require->{$_};
-                }
-
-                if (   $makefile =~ /ExtUtils::/
-                    && $self->name ne 'cpan-ExtUtils-MakeMaker' )
+            if (   $source =~ /ExtUtils::/
+                && $self->name ne 'cpan-ExtUtils-MakeMaker' )
+            {
+                unless ( defined $require->{requires}{'ExtUtils::MakeMaker'}
+                    && $require->{requires}{'ExtUtils::MakeMaker'} >= 6.31 )
                 {
-                    unless ( $require->{requires}{'ExtUtils::MakeMaker'}
-                        && $require->{requires}{'ExtUtils::MakeMaker'} >= 6.31 )
-                    {
-                        $require->{build_requires}{'ExtUtils::MakeMaker'} =
-                          6.31;
-                    }
+                    $require->{build_requires} =
+                      { 'ExtUtils::MakeMaker' => 6.31 };
                 }
             }
+
+#      # Makefile doesn't have recommends or build_requires stuff, we need to fix
+#      # that accroding to META.yml
+#            my $meta_path = File::Spec->catfile( $path, 'META.yml' );
+#            if ( -e $meta_path ) {
+#                my $meta = Shipwright::Util::LoadFile($meta_path);
+#
+#                for (qw/recommends build_requires/) {
+#                    my $keep = 'keep_' . $_;
+#                    if ( $self->$keep && $meta->{$_} && ! $require->{$_} ) {
+#                        $require->{$_} = $meta->{$_};
+#                    }
+#                }
+#            }
+
             Shipwright::Util->run( [ 'make', 'clean' ] );
-            Shipwright::Util->run( [ 'rm',   'Makefile.old' ] );
+            Shipwright::Util->run( [ 'rm', 'Makefile.old' ] );
         }
 
         for my $type (qw/requires recommends build_requires/) {
@@ -243,7 +141,7 @@ EOF
         }
 
         Shipwright::Util::DumpFile( $require_path, $require )
-          or confess "can't dump __require.yml: $!";
+          or die "can't dump __require.yml: $!";
     }
 
     if ( my $require = Shipwright::Util::LoadFile($require_path) ) {
@@ -279,17 +177,9 @@ EOF
                 }
 
                 my $name = $module;
-
                 if ( $self->_is_skipped($module) ) {
-                    unless ( defined $map->{$module}
-                        || defined $url->{$module} )
-                    {
-
-                        # not in the map, meaning it's not been imported before,
-                        # so it's safe to erase it
-                        delete $require->{$type}{$module};
-                        next;
-                    }
+                    delete $require->{$type}{$module}
+                      unless defined $map->{$module};
                 }
                 else {
 
@@ -322,19 +212,15 @@ EOF
                         {
                             $s = Shipwright::Source->new(
                                 %$self,
-                                source  => $require->{$type}{$module}{source},
-                                name    => $name,
-                                version => undef,
-                                _path   => undef,
+                                source => $require->{$type}{$module}{source},
+                                name   => $name,
                             );
                         }
                         else {
                             $s = Shipwright::Source->new(
                                 %$self,
-                                source  => "cpan:$module",
-                                version => undef,
+                                source => $module,
                                 name => '',   # cpan name is automaticaly fixed.
-                                _path   => undef,
                             );
                         }
                         $s->run();
@@ -345,10 +231,8 @@ EOF
                     if ( -e $self->map_path ) {
                         $map = Shipwright::Util::LoadFile( $self->map_path );
                     }
-
                 }
 
-                # convert required module name to dist name like cpan-Jifty-DBI
                 if ( $map->{$module} && $map->{$module} =~ /^cpan-/ ) {
                     $require->{$type}{ $map->{$module} } =
                       delete $require->{$type}{$module};
@@ -391,26 +275,13 @@ sub _update_url {
     my $url  = shift;
 
     my $map = {};
-    if ( -e $self->url_path && !-z $self->url_path ) {
+    if ( -e $self->url_path ) {
         $map = Shipwright::Util::LoadFile( $self->url_path );
     }
     $map->{$name} = $url;
     Shipwright::Util::DumpFile( $self->url_path, $map );
 }
 
-sub _update_version {
-    my $self    = shift;
-    my $name    = shift;
-    my $version = shift;
-
-    my $map = {};
-    if ( -e $self->version_path && !-z $self->version_path ) {
-        $map = Shipwright::Util::LoadFile( $self->version_path );
-    }
-    $map->{$name} = $version;
-    Shipwright::Util::DumpFile( $self->version_path, $map );
-}
-
 sub _is_skipped {
     my $self   = shift;
     my $module = shift;
@@ -432,7 +303,7 @@ sub _copy {
             my $cmd = [
                 'cp',
                 $file{$_},
-                catfile(
+                File::Spec->catfile(
                     $self->directory,
                     $self->name || $self->just_name( $self->path ), $_
                 )
@@ -451,49 +322,51 @@ trim the version stuff from dist name
 sub just_name {
     my $self = shift;
     my $name = shift;
-
-    $name =~ s/tar\.bz2$/tar.gz/;    # CPAN::DistnameInfo doesn't like bz2
-
-    $name .= '.tar.gz' unless $name =~ /(tar\.gz|tgz)$/;
-
-    require CPAN::DistnameInfo;
-    my $info = CPAN::DistnameInfo->new($name);
-    my $dist = $info->dist;
-    return $dist;
-}
-
-=head2 just_version
-
-return version
-
-=cut
-
-sub just_version {
-    my $self = shift;
-    my $name = shift;
     $name .= '.tar.gz' unless $name =~ /(tar\.gz|tgz|tar\.bz2)$/;
-
     require CPAN::DistnameInfo;
-    my $info    = CPAN::DistnameInfo->new($name);
-    my $version = $info->version;
-    $version =~ s/^v// if $version;
-    return $version;
-}
-
-=head2 is_compressed
-
-return true if the source is compressed file, i.e. tar.gz(tgz) and tar.bz2
-
-=cut
-
-sub is_compressed {
-    my $self = shift;
-    return 1 if $self->source =~ m{.*/.+\.(tar.(gz|bz2)|tgz)$};
-    return;
+    return CPAN::DistnameInfo->new($name)->dist;
 }
 
 1;
 
 __END__
 
-=head1 INTERFACE
+=head1 NAME
+
+Shipwright::Source::Base - base class for source
+
+
+=head1 SYNOPSIS
+
+  
+=head1 DESCRIPTION
+
+
+=head1 INTERFACE 
+
+
+=head1 DEPENDENCIES
+
+None.
+
+
+=head1 INCOMPATIBILITIES
+
+None reported.
+
+
+=head1 BUGS AND LIMITATIONS
+
+No bugs have been reported.
+
+=head1 AUTHOR
+
+sunnavy  C<< <sunnavy@bestpractical.com> >>
+
+
+=head1 LICENCE AND COPYRIGHT
+
+Copyright 2007 Best Practical Solutions.
+
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
