@@ -5,12 +5,9 @@ use strict;
 use Carp;
 
 use Shipwright::Source::Compressed;
-use File::Spec::Functions qw/catfile/;
 
 use base qw/Shipwright::Source::Base/;
 
-$ENV{'FTP_PASSIVE'} = 1;    # force enable passive mode, seems safe nowadays
-
 =head2 run
 
 =cut
@@ -38,11 +35,11 @@ sub _run {
         $self->_update_url( $self->just_name($file), $source );
         my $src_dir = $self->download_directory;
         mkdir $src_dir unless -e $src_dir;
-        $self->source( catfile( $src_dir, $file ) );
+        $self->source( File::Spec->catfile( $src_dir, $file ) );
 
         require LWP::UserAgent;
         my $ua = LWP::UserAgent->new;
-        $ua->timeout(1200);
+        $ua->timeout(10);
 
         my $response = $ua->get($source);
 
