@@ -3,7 +3,7 @@ package Shipwright::Source::HTTP;
 use warnings;
 use strict;
 use Carp;
-use File::Spec::Functions qw/catfile catdir/;
+use File::Spec;
 use Shipwright::Source::Compressed;
 
 use base qw/Shipwright::Source::Base/;
@@ -38,11 +38,11 @@ sub _run {
 
         my $src_dir = $self->download_directory;
         mkdir $src_dir unless -e $src_dir;
-        $self->source( catfile( $src_dir, $file ) );
+        $self->source( File::Spec->catfile( $src_dir, $file ) );
 
         require LWP::UserAgent;
         my $ua = LWP::UserAgent->new;
-        $ua->timeout(1200);
+        $ua->timeout(10);
 
         my $response = $ua->get($source);
 
